import { describe, it, expect } from 'vitest'
import * as rollup from 'rollup'
import gettextPlugin from '../dist/index'

describe('compile po file', () => {
  async function render (opts = {}) {
    const bundle = await rollup.rollup({
      input: {
        'test/mySimpleModule': 'gettext'
      },
      plugins: [gettextPlugin(Object.assign({
        poFiles: 'spec/fixtures/i18n/*.po',
        defaultDictionary: 'test/mySimpleModule',
        defaultLanguage: 'de_DE'
      }, opts))]
    })
    return await bundle.generate({})
  }

  function getTranslationsFrom (mod) {
    return JSON.parse(mod.code.match(/ = ({\n.*headers:\s*headers,[^}]+})/m)[1].replace(/headers:\s*headers,/, ''))
  }

  describe('using a simple example', () => {
    it('should generate a loadable ES module', async () => {
      const { output } = await render()
      const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
      const alternativeModule = output.find(chunk => /alternative/.test(chunk.fileName))
      expect(simpleModule.code.match(/Language:\s*"([^"]+)/)[1]).toBe('de_DE')
      expect(alternativeModule.code.match(/Language:\s*"([^"]+)/)[1]).toBe('de_DE')
    })

    it('missing translation should be filtered from dictionary', async () => {
      const { output } = await render()
      expect(output.map(c => c.code).join('\n').indexOf('part of computer\\u0000Drive contains %1$d item\\u0001Drive contains %1$d items')).toBeLessThan(0)
    })

    describe('items with no context', () => {
      it('should render keys with singular form only', async () => {
        const { output } = await render()
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        expect(translation['\x00translate me!\x01'][0]).toBe('übersetze mich!')
      })

      it('should render keys with singular and plural forms', async () => {
        const { output } = await render()
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        expect(translation['\x00Box contains %1$d item\x01Box contains %1$d items'][0]).toBe('Box enthält %1$d Elemente')
        expect(translation['\x00Box contains %1$d item\x01Box contains %1$d items'][1]).toBe('Box enthält %1$d Elemente')
      })

      it('should render keys with singualar and plural forms with only one field specified in po file', async () => {
        const { output } = await render()
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        const result = translation['\x00Partially translated plural string\x01Partially translated plural strings']
        expect(result).toBeInstanceOf(Array)
        expect(result.length).toBe(1)
        expect(result[0]).toBe('Teilweise übersetzte Pluralzeichenkette')
      })
    })

    describe('items with context', () => {
      it('should render keys with singular form only', async () => {
        const { output } = await render()
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        expect(translation['describe state\x00Empty folder\x01'][0]).toBe('Leerer Ordner')
        expect(translation['perform action\x00Empty folder\x01'][0]).toBe('Leere Ordner')
      })

      it('should render keys with singular and plural forms', async () => {
        const { output } = await render()
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        expect(translation['\x00Box contains %1$d item\u0001Box contains %1$d items'][0]).toBe('Box enthält %1$d Elemente')
        expect(translation['\x00Box contains %1$d item\u0001Box contains %1$d items'][1]).toBe('Box enthält %1$d Elemente')
      })
    })

    describe('messages with fuzzy flag', () => {
      it('should be filtered from catalog by default', async () => {
        const { output } = await render()
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        expect(translation['\x00A message with fuzzy translation\x01']).toBeUndefined()
      })

      it('should be included in catalog if includeFuzzy option is set', async () => {
        const { output } = await render({ includeFuzzy: true })
        const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
        const translation = getTranslationsFrom(simpleModule)
        expect(translation['\x00A message with fuzzy translation\x01']).toBeInstanceOf(Array)
      })

      it('should dynamically load translation modules', async () => {
        const bundle = await rollup.rollup({
          input: 'test/mySimpleModule',
          plugins: [gettextPlugin({
            poFiles: 'spec/fixtures/i18n/*.po',
            defaultDictionary: 'test/mySimpleModule',
            defaultLanguage: 'de_DE'
          })]
        })
        expect((await bundle.generate({})).output.map(c => c.fileName)).toHaveLength(4)
      })
    })
  })

  describe('with messages pointing to different modules', () => {
    it('should put message into each referenced module', async () => {
      const { output } = await render()
      const simpleModule = output.find(chunk => /SimpleModule/.test(chunk.fileName))
      const translationSimpleModule = getTranslationsFrom(simpleModule)
      const alternativeModule = output.find(chunk => /alternative/.test(chunk.fileName))
      const translationAlternativeModule = getTranslationsFrom(alternativeModule)
      expect(translationSimpleModule['\x00String with " in it\x01']).toBeInstanceOf(Array)
      expect(translationAlternativeModule['\x00String with " in it\x01']).toBeInstanceOf(Array)
    })
  })

  it('should not extract obsolete messages', async () => {
    const { output } = await render()
    const alternativeModule = output.find(chunk => /alternative/.test(chunk.fileName))
    const translation = getTranslationsFrom(alternativeModule)
    expect(translation['\x00unused message\x01']).not.toBeInstanceOf(Array)
  })
})
