import { describe, it, expect } from 'vitest'
import * as rollup from 'rollup'
import gettextPlugin from '../dist/index'
import { findStrings } from './utils'

describe('extract comments', () => {
  describe('simple example files', () => {
    async function renderPo () {
      const bundle = await rollup.rollup({
        input: 'spec/fixtures/simple_example.js',
        external: [
          'randomModule',
          'randomModule2'
        ],
        plugins: [
          gettextPlugin({
            poFiles: 'spec/fixtures/i18n/*.po',
            defaultLanguage: 'de_DE'
          })
        ]
      })
      return bundle.cache.plugins.gettext.po[1]
    }

    it('should extract gettext flags', async () => {
      const po = await renderPo()
      const [item] = findStrings('Welcome, %1$s %2$s!', po.items)
      expect(item.flags['c-format']).toBe(true)
    })

    it('should extract plain comments for translators (plain gt call)', async () => {
      const po = await renderPo()

      const [item] = findStrings('Welcome, %1$s %2$s!', po.items)
      // comments extracted
      expect(item.extractedComments).toContain('%1$s is the given name')
      expect(item.extractedComments).toContain('%2$s is the family name')
    })

    it('should extract comments inside objects', async () => {
      const po = await renderPo()

      const [item] = findStrings('Where is Banana Joe', po.items)
      // comments extracted
      expect(item.extractedComments).toEqual([
        'Banana joe',
        'You know'
      ])
    })

    it('should extract plain comments for translators (gt.pgettext call)', async () => {
      const po = await renderPo()

      const [item1, item2] = findStrings('Empty folder', po.items)
      // comments extracted
      expect(item1.extractedComments).toEqual(['empty is meant as an action (make folder empty!)'])
      expect(item2.extractedComments).toEqual(['empty is meant as a state (folder is empty)'])
    })

    it('should extract plain comments for translators (gt.ngettext call)', async () => {
      const po = await renderPo()

      const [item] = findStrings('Box contains %1$d item', po.items)
      // comments extracted
      expect(item.extractedComments).toContain('%1$d is the number of items in the box')
    })

    it('should extract plain comments for translators (gt.npgettext call)', async () => {
      const po = await renderPo()

      const [item] = findStrings('Drive contains %1$d item', po.items)
      // comments extracted
      expect(item.extractedComments).toContain('Drive is meant to be a product name, here')
    })

    it('should extract comments in the same line as the gt call', async () => {
      const po = await renderPo()

      let [item] = findStrings('B', po.items)
      expect(item.extractedComments).toEqual(['another gt comment for Bytes', 'Bytes'])

      item = findStrings('GB', po.items)[0]
      expect(item.extractedComments).toEqual(['Gigabytes'])

      item = findStrings('YB', po.items)[0]
      expect(item.extractedComments).toEqual(['Yottabytes'])
    })

    it('should not extract exactly the same comment from different file twice', async () => {
      const po = await renderPo()

      const [item] = findStrings('Welcome, %1$s %2$s!', po.items)
      expect(item.extractedComments.filter(comment => comment === '%1$s is the given name'))
        .toHaveLength(1)
      expect(item.extractedComments.filter(comment => comment === '%2$s is the family name'))
        .toHaveLength(1)
    })
  })
})
