import { describe, it, expect } from 'vitest'

import * as rollup from 'rollup'
import gettextPlugin from '../dist/index'
import { findStrings } from './utils'

describe('Extract messages', () => {
  async function renderPo () {
    const bundle = await rollup.rollup({
      input: 'spec/fixtures/simple_example.js',
      external: [
        'randomModule',
        'randomModule2'
      ],
      plugins: [
        gettextPlugin({
          poFiles: 'spec/fixtures/i18n/*.po',
          defaultDictionary: 'test/mySimpleModule',
          defaultLanguage: 'de_DE'
        })
      ]
    })
    return bundle.cache.plugins.gettext.po[1]
  }

  describe('from simple example files', () => {
    it('should extract plain gt calls', async () => {
      const po = await renderPo()
      const [item] = findStrings('translate me!', po.items)
      expect(item).toBeDefined()
      expect(item.references).toContain('spec/fixtures/simple_example.js')
    })

    it('should extract plural forms', async () => {
      const po = await renderPo()
      const [item] = findStrings('Box contains %1$d item', po.items)
      expect(item).toBeDefined()
      expect(item.msgid_plural).toBe('Box contains %1$d items')
    })

    it('should extract plural forms with context', async () => {
      const po = await renderPo()
      const items = findStrings('Drive contains %1$d item', po.items)
      expect(items).toHaveLength(2)
      expect(items[0].msgid_plural).toBe('Drive contains %1$d items')
      expect(items[1].msgid_plural).toBe('Drive contains %1$d items')
      expect(items.map(item => item.msgctxt)).toContain('product name')
      expect(items.map(item => item.msgctxt)).toContain('part of computer')
    })

    it('should extract message context', async () => {
      const po = await renderPo()
      const items = findStrings('Empty folder', po.items)
      expect(items).toHaveLength(2)
      expect(items.map(item => item.msgctxt)).toContain('perform action')
      expect(items.map(item => item.msgctxt)).toContain('describe state')
    })
  })

  describe('from fuzzy JavaScript modules', () => {
    it('should work for simple strings', async () => {
      const po = await renderPo()
      const [item] = findStrings('A string from a fuzzy module', po.items)
      expect(item).toBeDefined()
    })

    it('should escape linebreaks', async () => {
      const po = await renderPo()
      const [item] = findStrings('A string with\n \\n in it', po.items)
      expect(item).toBeDefined()
    })

    it('should find string within gt.format call', async () => {
      const po = await renderPo()
      const [item] = findStrings('%1$d Minute', po.items)
      expect(item.msgid_plural).toBe('%1$d Minutes')
    })

    it('should not trim msgids', async () => {
      const po = await renderPo()
      const [item] = findStrings('White-space hurray! ', po.items)
      expect(item).toBeDefined()
    })
  })

  describe('statically concatenated strings', () => {
    it('should be detected in plain gt calls', async () => {
      const po = await renderPo()
      const [item] = findStrings('A custom concatenated string to support multiple lines', po.items)
      expect(item).toBeDefined()
    })

    it('should be detected in gt.pgettext calls', async () => {
      const po = await renderPo()
      const [item] = findStrings('concat pgettext', po.items)
      expect(item.msgid).toBe('concat pgettext')
      expect(item.msgctxt).toBe('my context')
    })

    it('should be detected in gt.ngettext calls', async () => {
      const po = await renderPo()
      const [item] = findStrings('%1$d thing', po.items)
      expect(item).toBeDefined()
    })

    it('should be detected in gt.npgettext calls', async () => {
      const po = await renderPo()
      const [item] = findStrings('with %1$d thing', po.items)
      expect(item.msgid).toBe('with %1$d thing')
      expect(item.msgctxt).toBe('my context')
    })
  })

  describe('auto-detection of c-format strings', () => {
    it('should add flag for %1$s', async () => {
      const po = await renderPo()
      const [item] = findStrings('A c-format string %1$s', po.items)
      expect(item.flags['c-format']).toBe(true)
    })

    it('shouldn\'t add flag for non-c-format strings', async () => {
      const po = await renderPo()
      const [item] = findStrings('String with " in it', po.items)
      expect(item.flags['c-format']).toBe(undefined)
    })
  })
})
