import { posix as path } from 'node:path'
import { parse, ImportSpecifier } from 'es-module-lexer'
import MagicString from 'magic-string'
import type { Plugin, UserConfig, ConfigEnv } from 'vite'
import type { OutputChunk, OutputBundle } from 'rollup'

export interface VitePluginOxExternalsOptions {
  /**
   * The path prefix to be used to import external modules.
   * @default "$"
   */
  prefix?: string
}

export interface VitePluginOxExternals extends Plugin {
  meta: Required<VitePluginOxExternalsOptions>
}

export const PROJECT_NAME = '@open-xchange/vite-plugin-ox-externals'

/**
 * Escapes special characters in a string to create a valid regular expression pattern.
 * @param string - The string to escape
 * @returns The escaped string safe for use in a RegExp
 */
function escapeRegex (string: string): string {
  return string.replace(/[-/\\^$*+?.()|[\]{}]/g, '\\$&')
}

/**
 * Creates a Vite plugin that handles external OX Core modules by transforming
 * prefixed imports into proper external module paths.
 *
 * @example
 * ```ts
 * // vite.config.ts
 * import vitePluginOxExternals from '@open-xchange/vite-plugin-ox-externals'
 *
 * export default defineConfig({
 *   plugins: [
 *     vitePluginOxExternals({ prefix: '$' })
 *   ]
 * })
 * ```
 *
 * @param options - Configuration options for the plugin
 * @returns A Vite plugin instance
 */
export default function vitePluginOxExternals (
  options?: VitePluginOxExternalsOptions
): VitePluginOxExternals {
  const resolvedOptions: Required<VitePluginOxExternalsOptions> = {
    prefix: '$',
    ...options
  }

  const prefixRE = new RegExp(`^${escapeRegex(resolvedOptions.prefix)}\\/(.*?)(\\.js|\\.vue|)$`)
  let mode: string
  const externalSet = new Set<string>()

  /**
   * Checks if a module ID is marked as external.
   * @param id - The module ID to check
   * @returns True if the module is external, false otherwise
   */
  function isExternalModule (id: string): boolean {
    return externalSet.has(id)
  }

  /**
   * Resolves the external path for a given module path, handling extensions
   * and development mode paths appropriately.
   *
   * @param path - The module path to resolve
   * @returns The resolved external path with proper extension
   */
  function resolveExternalPath (path: string): string {
    const extension = path.endsWith('.vue') ? 'vue' : 'js'
    return `${mode === 'development' ? '/' : ''}${path}.${extension}`
  }

  /**
   * Processes imports within a chunk, replacing external module paths with
   * their relative equivalents.
   *
   * @param chunk - The chunk containing code and filename
   * @param s - MagicString instance for code manipulation
   * @param imports - Array of import positions from es-module-lexer
   */
  function processChunkImports (
    chunk: OutputChunk,
    s: MagicString,
    imports: readonly ImportSpecifier[]
  ): void {
    for (const { s: start, e: end } of imports) {
      const importStatement = chunk.code.substring(start, end)
      const matches = importStatement.match(/(["'`]?)([^"'`]*)(["'`]?)/)
      if (!matches) continue

      const [, quote, match, endQuote] = matches
      if (!isExternalModule(match)) continue

      const relativePath = path.relative(
        path.dirname(`/${chunk.fileName}`),
        `/${match}`
      )
      s.overwrite(start, end, `${quote}${relativePath}${endQuote}`)
    }
  }

  /**
   * Configures the plugin based on the Vite environment.
   */
  function configurePlugin (config: UserConfig, env: ConfigEnv) {
    mode = env.mode

    if (mode === 'development') {
      config.build ??= {}
      config.build.rollupOptions ??= {}
      config.build.rollupOptions.input = {}
    }
  }

  /**
   * Resolves module IDs that match the configured prefix pattern.
   */
  function resolveModuleId (id: string) {
    if (!id.startsWith(resolvedOptions.prefix + '/')) return null

    const match = prefixRE.exec(id)
    if (!match) return null

    const [, path] = match
    const newId = resolveExternalPath(path)

    externalSet.add(newId)
    return { id: newId, external: true }
  }

  /**
   * Loads external modules.
   */
  function loadModule (id: string) {
    return isExternalModule(id) ? '' : null
  }

  /**
   * Handles bundle generation and import path transformations.
   */
  function generateModuleBundle (_options: unknown, bundle: OutputBundle) {
    for (const file in bundle) {
      const chunk = bundle[file]
      if (chunk.type !== 'chunk') continue

      const allImports = new Set([
        ...chunk.imports,
        ...chunk.dynamicImports
      ])

      if (!Array.from(allImports).some(isExternalModule)) continue

      const s = new MagicString(chunk.code)
      const [imports] = parse(chunk.code)

      processChunkImports(chunk, s, imports)
      chunk.code = s.toString()
    }
  }

  return {
    name: PROJECT_NAME,
    meta: resolvedOptions,
    enforce: 'pre',
    config: configurePlugin,
    resolveId: resolveModuleId,
    load: loadModule,
    generateBundle: generateModuleBundle
  }
}
