import type net from 'node:net'
import type http from 'node:http'
import http2Client from 'http2-client'
import type { ViteDevServer } from 'vite'

import type { DevServerConfigData, FrontendConfig, PluginMiddleware } from '../utils/types.js'
import logger from '../utils/logger.js'

function getAddressInfo (server: http.Server): Partial<net.AddressInfo> {
  const info = server.address()
  return (typeof info === 'string') ? { address: info } : (info || {})
}

function getHostFromServer (server: http.Server): string {
  const { address = '', family } = getAddressInfo(server)
  if (family === 'IPv6' && address === '::') {
    logger.info('Replace "::" with "0.0.0.0"')
    return '0.0.0.0'
  }
  if (family === 'IPv6') logger.warn(`Detected IPv6 address ${address}. This might not work.`)
  return address
}

function getPortFromServer (server: http.Server): number {
  return getAddressInfo(server).port || 80
}

function ping (url: URL, timeout: number): Promise<unknown> {
  return Promise.race([
    new Promise((resolve, reject) => {
      const { protocol, hostname, port } = url
      logger.info(`Send ping request to ${url.href}`)
      http2Client.request({
        protocol,
        hostname,
        port,
        path: '/ping',
        headers: {
          accept: '*/*',
          'ping-by-ox-proxy': 'true'
        }
      }, res => {
        if (res.statusCode !== 200) return reject(new Error('Not found'))
        const result: string[] = []
        res.on('data', (data: Buffer) => {
          result.push(data.toString())
        })
        res.on('end', (data: string | Buffer = '') => {
          result.push(data.toString())
          if (res.headers['content-type'] === 'application/json') {
            return resolve(JSON.parse(result.join('')))
          }
          resolve(result.join(''))
        })
      }).end()
    }),
    new Promise((_resolve, reject) => {
      setTimeout(() => reject(new Error('Request timed out')), timeout)
    })
  ])
}

let devServerUrl: URL

export function getDevServerURL (): URL {
  return devServerUrl
}

interface HTTPServer extends http.Server {
  isClosed?: true
}

async function findDevServer (httpServer: HTTPServer, protocol: string, hostnames: string[], port: number): Promise<FrontendConfig | void> {
  for (const host of hostnames) {
    try {
      const url = new URL(`${protocol}//${host}:${port}`)
      logger.trace(`Constructed url: "${url.href}". Let's see if its valid.`)
      await ping(url, 100)
      logger.info(`Detected dev-server on ${url.href}`)
      // TODO we might want to get additional configuration
      return { url, secure: false }
    } catch {
      if (httpServer.isClosed) return
      logger.warn(`Could not verify url: ${protocol}//${host}:${port}`)
    }
  }
}

export default function detectDevServerMiddleware (server: ViteDevServer, frontends: FrontendConfig[]): PluginMiddleware {
  const httpServer = server.httpServer as HTTPServer | null // typecast for additional property "isClosed"
  if (!httpServer) throw new Error('missing HTTP server')

  httpServer.once('close', () => {
    httpServer.isClosed = true
  })

  const setupPromise = new Promise<void>(resolve => {
    httpServer.once('listening', async () => {
      const protocol = server.config.server.https ? 'https:' : 'http:'
      const hostname = getHostFromServer(httpServer)
      const port = getPortFromServer(httpServer)
      const hostnames = Array.from(new Set([hostname, '127.0.0.1', 'localhost', '0.0.0.0']))

      // retry 5 times
      for (let i = 0; i < 5; i++) {
        const frontend = await findDevServer(httpServer, protocol, hostnames, port)
        if (frontend) {
          devServerUrl = frontend.url
          frontends.unshift(frontend)
          logger.trace(`Added ${frontend.url.href} to frontends.`)
          return resolve()
        }
        if (httpServer.isClosed) return resolve()
        logger.trace(`Dev-server could not be detected. Try all hostnames "${hostnames.join(', ')}" again.`)
      }

      logger.fatal('Could not find a working local vite dev server.')
      if (!httpServer.isClosed) process.exit(1)
    })
  }).then(async () => {
    for (const frontend of frontends) {
      const result = await ping(frontend.url, 100).catch(() => logger.debug(`${frontend.url.href} is not a server with vite-plugin-proxy.`))
      if (result) {
        frontend.config = result as DevServerConfigData
        logger.trace(`Got config from ${frontend.url.href}: ${JSON.stringify(result)}`)
      }
    }
  })

  return async function (req, res, next) {
    // respond to ping requests
    if (req.headers['ping-by-ox-proxy']) {
      logger.info('Respond ping request')
      res.writeHead(200, { 'Content-Type': 'application/json' })
      const data: DevServerConfigData = { fs: { allow: server.config.server.fs.allow }, root: server.config.root, base: server.config.base }
      res.end(JSON.stringify(data))
      return
    }

    await setupPromise
    next()
  }
}
