import { describe, it, expect, afterEach, vi } from 'vitest'
import vitePluginProxy from '../../dist/index'
import { createTestServer, filterPingCalls, generateSelfsigned, getPort, http2Request, updateRequestManager } from '../util'
import { createServer } from 'vite'
import path from 'node:path'
const __dirname = import.meta.dirname
process.noDeprecation = true

describe('enhanced frontend configuration', () => {
  afterEach(async () => {
    updateRequestManager()
  })

  it('select a preferred frontend by path', async () => {
    const server = await createTestServer()
    const mockCallback = vi.fn((req, res) => res.send('alert("foo")'))
    server.get('*', mockCallback)
    const SERVER_PORT = await getPort()
    await server.listen({ port: SERVER_PORT })

    const server2 = await createTestServer()
    const mockCallback2 = vi.fn((req, res) => res.send('alert("bar")'))
    server2.get('*', mockCallback2)
    const SERVER_PORT2 = await getPort()
    await server2.listen({ port: SERVER_PORT2 })

    const server1 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT}`,
          path: '/core'
        }, {
          target: `http://localhost:${SERVER_PORT2}`,
          path: '/plugin'
        }]
      })]
    })
    const PORT = await getPort()
    await server1.listen(PORT)

    expect(filterPingCalls(mockCallback)).toHaveBeenCalledTimes(0)
    expect(filterPingCalls(mockCallback2)).toHaveBeenCalledTimes(0)

    const { data } = await http2Request(`http://localhost:${PORT}/core/file.js`)
    expect(data).toBe('alert("foo")')
    expect(filterPingCalls(mockCallback)).toHaveBeenCalledTimes(1)
    expect(filterPingCalls(mockCallback2)).toHaveBeenCalledTimes(0)

    const { data: data2 } = await http2Request(`http://localhost:${PORT}/plugin/file.js`)
    expect(data2).toBe('alert("bar")')
    expect(filterPingCalls(mockCallback)).toHaveBeenCalledTimes(1)
    expect(filterPingCalls(mockCallback2)).toHaveBeenCalledTimes(1)

    server2.close()
    server1.close()
    server.close()
  })

  it('select a preferred frontend by regex', async () => {
    const server = await createTestServer()
    const mockCallback = vi.fn((req, res) => res.send('alert("foo")'))
    server.get('*', mockCallback)
    const SERVER_PORT = await getPort()
    await server.listen({ port: SERVER_PORT })

    const server2 = await createTestServer()
    const mockCallback2 = vi.fn((req, res) => res.send('alert("bar")'))
    server2.get('*', mockCallback2)
    const SERVER_PORT2 = await getPort()
    await server2.listen({ port: SERVER_PORT2 })

    const server3 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT}`,
          path: '^/core'
        }, {
          target: `http://localhost:${SERVER_PORT2}`,
          path: '^/(?!core)'
        }]
      })]
    })

    const PORT = await getPort()
    await server3.listen(PORT)

    expect(filterPingCalls(mockCallback)).toHaveBeenCalledTimes(0)
    expect(filterPingCalls(mockCallback2)).toHaveBeenCalledTimes(0)

    const { data } = await http2Request(`http://localhost:${PORT}/core/file.js`)
    expect(data).toBe('alert("foo")')
    expect(filterPingCalls(mockCallback)).toHaveBeenCalledTimes(1)
    expect(filterPingCalls(mockCallback2)).toHaveBeenCalledTimes(0)

    const { data: data2 } = await http2Request(`http://localhost:${PORT}/plugin/file.js`)
    expect(data2).toBe('alert("bar")')
    expect(filterPingCalls(mockCallback)).toHaveBeenCalledTimes(1)
    expect(filterPingCalls(mockCallback2)).toHaveBeenCalledTimes(1)
    await server.close()
    await server2.close()
    await server3.close()
  })

  it('supports secure=false', async () => {
    const server = await createTestServer({
      http2: true,
      https: generateSelfsigned()
    })
    server.get('*', (req, res) => res.send('alert("foo")'))
    const SERVER_PORT = await getPort()
    await server.listen({ port: SERVER_PORT })

    const server2 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `https://localhost:${SERVER_PORT}`,
          secure: false
        }]
      })]
    })
    const PORT = await getPort()
    await server2.listen(PORT)

    await http2Request(`http://localhost:${PORT}/plugin/file.js`)

    await server2.close()
    await server.close()
  })

  it('supports frontends with different base configurations', async () => {
    const server2 = await createServer({
      root: path.join(__dirname, 'appsuite'),
      logLevel: 'silent',
      base: '/appsuite/',
      plugins: [vitePluginProxy()]
    })
    const SERVER_PORT2 = await getPort()
    await server2.listen(SERVER_PORT2)

    const server1 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT2}`
        }]
      })]
    })
    const SERVER_PORT1 = await getPort()
    await server1.listen(SERVER_PORT1)

    const { data } = await http2Request(`http://localhost:${SERVER_PORT1}/index.js`)
    expect(data).toMatch('window.test = \'foo\'\n')

    const { data: data2 } = await http2Request(`http://localhost:${SERVER_PORT1}/bar.js`)
    expect(data2).toMatch('window.test = \'bar\'\n')
    await server1.close()
    await server2.close()
  })

  it('supports frontends with different base configurations (2)', async () => {
    const server2 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/',
      server: {
        fs: {
          allow: ['/']
        }
      },
      plugins: [vitePluginProxy()]
    })
    const SERVER_PORT2 = await getPort()
    await server2.listen(SERVER_PORT2)

    const server1 = await createServer({
      root: path.join(__dirname, 'appsuite'),
      logLevel: 'silent',
      base: '/appsuite/',
      server: {
        fs: {
          allow: ['/']
        }
      },
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT2}`
        }]
      })]
    })
    const SERVER_PORT1 = await getPort()
    await server1.listen(SERVER_PORT1)

    const { data } = await http2Request(`http://localhost:${SERVER_PORT1}/appsuite/bar.js`)
    expect(data).toMatch('window.test = \'bar\'\n')

    const { data: data2 } = await http2Request(`http://localhost:${SERVER_PORT1}/appsuite/index.js`)
    expect(data2).toMatch('window.test = \'foo\'\n')
    await server1.close()
    await server2.close()
  })

  it('supports frontends with different base configurations (3)', async () => {
    const server2 = await createServer({
      root: path.join(__dirname, 'appsuite'),
      logLevel: 'silent',
      base: '/appsuite/',
      plugins: [vitePluginProxy()]
    })
    const SERVER_PORT2 = await getPort()
    await server2.listen(SERVER_PORT2)

    const server1 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/appsuite/',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT2}/appsuite/`
        }]
      })]
    })
    const SERVER_PORT1 = await getPort()
    await server1.listen(SERVER_PORT1)

    const { data } = await http2Request(`http://localhost:${SERVER_PORT1}/appsuite/index.js`)
    expect(data).toMatch('window.test = \'foo\'\n')

    const { data: data2 } = await http2Request(`http://localhost:${SERVER_PORT1}/appsuite/bar.js`)
    expect(data2).toMatch('window.test = \'bar\'\n')

    await server1.close()
    await server2.close()
  })

  it('proxies requests to frontends that allow file system access', async () => {
    const server2 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/',
      plugins: [vitePluginProxy()]
    })
    const SERVER_PORT2 = await getPort()
    await server2.listen(SERVER_PORT2)

    const server1 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT2}`
        }]
      })]
    })
    const PORT = await getPort()
    await server1.listen(PORT)

    const { data } = await http2Request(`http://localhost:${PORT}/@fs${__dirname}/index.js`)
    expect(data).toMatch('window.test = \'foo\'\n')
    const { data: data2 } = await http2Request(`http://localhost:${PORT}/@fs${__dirname}/appsuite/bar.js`)
    expect(data2).toMatch('window.test = \'bar\'\n')
    await server1.close()
    await server2.close()
  })

  it('proxies requests to frontends with base configured that allow file system access', async () => {
    const server2 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/appsuite/',
      plugins: [vitePluginProxy()]
    })
    const SERVER_PORT2 = await getPort()
    await server2.listen(SERVER_PORT2)

    const server1 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/appsuite/',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT2}/appsuite/`
        }]
      })]
    })
    const PORT = await getPort()
    await server1.listen(PORT)

    const { data } = await http2Request(`http://localhost:${PORT}/appsuite/@fs${__dirname}/index.js`)
    expect(data).toMatch('window.test = \'foo\'\n')
    const { data: data2 } = await http2Request(`http://localhost:${PORT}/appsuite/@fs${__dirname}/appsuite/bar.js`)
    expect(data2).toMatch('window.test = \'bar\'\n')
    await server1.close()
    await server2.close()
  })

  it('proxies requests to frontends with different bases that allow file system access', async () => {
    const server2 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/appsuite/',
      plugins: [vitePluginProxy({})]
    })
    const SERVER_PORT2 = await getPort()
    await server2.listen(SERVER_PORT2)

    const server1 = await createServer({
      root: __dirname,
      logLevel: 'silent',
      base: '/',
      plugins: [vitePluginProxy({
        frontends: [{
          target: `http://localhost:${SERVER_PORT2}`
        }]
      })]
    })
    const PORT = await getPort()
    await server1.listen(PORT)

    const { data } = await http2Request(`http://localhost:${PORT}/@fs${__dirname}/index.js`)
    expect(data).toMatch('window.test = \'foo\'\n')
    const { data: data2 } = await http2Request(`http://localhost:${PORT}/@fs${__dirname}/appsuite/bar.js`)
    expect(data2).toMatch('window.test = \'bar\'\n')
    await server1.close()
    await server2.close()
  })
})
