import { describe, it, expect, afterEach } from 'vitest'
import vitePluginProxy from '../../dist/index'
import { createTestServer, generateSelfsigned, getPort, http2Request, updateRequestManager } from '../util'
import { createServer } from 'vite'
const __dirname = import.meta.dirname
process.noDeprecation = true

describe('changeOrigin', () => {
  afterEach(async () => {
    updateRequestManager()
  })

  describe('http1 to http1', () => {
    it('is enabled', async () => {
      const server = await createTestServer()
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      const SERVER_PORT = await getPort()
      await server.listen({ port: SERVER_PORT })

      const server1 = await createServer({
        root: __dirname,
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `http://localhost:${SERVER_PORT}`,
              changeOrigin: true
            }
          }
        })]
      })
      const PORT = await getPort()
      await server1.listen(PORT)

      await http2Request(`http://localhost:${PORT}/api/test`)
      const headers = await promise
      expect(headers.host).toBe(`localhost:${SERVER_PORT}`)

      server1.close()
      server.close()
    })

    it('is disabled (default)', async () => {
      const server = await createTestServer()
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      const SERVER_PORT = await getPort()
      await server.listen({ port: SERVER_PORT })

      const server1 = await createServer({
        root: __dirname,
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `http://localhost:${SERVER_PORT}`
            }
          }
        })]
      })
      const PORT = await getPort()
      await server1.listen(PORT)

      await http2Request(`http://localhost:${PORT}/api/test`)
      const headers = await promise
      expect(headers.host).toBe(`localhost:${PORT}`)
      await server1.close()
      await server.close()
    })
  })

  describe('http2 to http1', () => {
    it('is enabled', async () => {
      const server = await createTestServer({
        http2: true,
        https: generateSelfsigned()
      })
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      const SERVER_PORT = await getPort()
      await server.listen({ port: SERVER_PORT })

      const server1 = await createServer({
        root: __dirname,
        server: { https: generateSelfsigned() },
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `https://localhost:${SERVER_PORT}`,
              changeOrigin: true,
              secure: false
            }
          }
        })]
      })
      const PORT = await getPort()
      await server1.listen(PORT)

      await http2Request(`https://localhost:${PORT}/api/test`, {
        rejectUnauthorized: false
      })
      const headers = await promise
      expect(headers.host).toBe(`localhost:${SERVER_PORT}`)
      await server1.close()
      await server.close()
    })

    it('is disabled (default)', async () => {
      const server = await createTestServer({
        http2: true,
        https: generateSelfsigned()
      })
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      const SERVER_PORT = await getPort()
      await server.listen({ port: SERVER_PORT })
      const server1 = await createServer({
        root: __dirname,
        server: { https: generateSelfsigned() },
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `https://localhost:${SERVER_PORT}`,
              secure: false
            }
          }
        })]
      })
      const PORT = await getPort()
      await server1.listen(PORT)

      await http2Request(`https://localhost:${PORT}/api/test`, {
        rejectUnauthorized: false
      })
      const headers = await promise
      expect(headers.host).toBe(`localhost:${PORT}`)
      await server1.close()
      await server.close()
    })
  })
})
